import asyncio
import json
from pathlib import Path
import numpy as np
import pandas as pd


class DataCollect(object):

    def __init__(self, save_dir):
        self.save_dir = save_dir
        self.csv_files: list[Path] = self.get_all_csv()

    def get_all_csv(self):
        return [sub_file for sub_file in Path(self.save_dir).iterdir() if
                sub_file.is_file() and sub_file.suffix == ".csv"]

    @staticmethod
    async def csv2json(file_path):
        df = await asyncio.to_thread(pd.read_csv, file_path)
        return json.loads(df.to_json(orient='records', double_precision=4))

    async def get_all_data(self, is_format=True):
        result = await asyncio.gather(*[self.csv2json(file.resolve()) for file in self.csv_files])
        all_data = [{"name": file.stem, "value": result[index]} for index, file in enumerate(self.csv_files)]
        if is_format:
            return self.format_all_data_value(all_data)

    @staticmethod
    def format_all_data_value(all_data: list[dict]):
        start_time = min([data.get("value")[0].get("time") for data in all_data if data.get("value")])
        end_time = max([data.get("value")[-1].get("time") for data in all_data if data.get("value")])
        # 补全开头结尾, 这里是计算密集型
        for data in all_data:
            format_all_data_dict = {cur_time: {"time": cur_time} for cur_time in range(start_time, end_time + 1)}
            if data.get("value"):
                old_value = data.get("value")
                for value in old_value:
                    format_all_data_dict[value.get("time")] = value
            data["value"] = list(format_all_data_dict.values())
        return all_data

    # numpy 增强版性能表现更好
    @staticmethod
    def format_all_data_value(all_data):
        start_time = min([data.get("value")[0].get("time") for data in all_data if data.get("value")])
        end_time = max([data.get("value")[-1].get("time") for data in all_data if data.get("value")])
        # 生成时间序列
        all_times = np.arange(start_time, end_time + 1)
        # 遍历所有数据
        for data in all_data:
            if 'value' in data:
                # 将原始数据的时间转换为数组
                original_times = np.array([item['time'] for item in data['value']])
                # 找出缺失的时间点
                missing_times = np.setdiff1d(all_times, original_times)
                # 为缺失的时间点创建新的对象
                missing_data = [{'time': int(time)} for time in missing_times]
                # 将新创建的对象添加到原始数据中
                data['value'].extend(missing_data)
                # 确保数据按照时间排序
                data['value'].sort(key=lambda x: x['time'])
        return all_data



import base64
import multiprocessing
import os
import platform
import shutil
import threading
import time
import traceback
import webbrowser

import psutil
from fastapi import FastAPI
from starlette.requests import Request
from starlette.responses import JSONResponse, RedirectResponse
from starlette.staticfiles import StaticFiles
from core.pc_tools import sys_info, pids, screenshot
from dao import TaskCollection
from log import log as logger
from task_handle import TaskHandle
from util import DataCollect


app = FastAPI()
logger.info("工作空间{0}".format(os.getcwd()))
BASE_CSV_DIR = os.path.join(os.path.dirname(__file__), "test_result")
if not os.path.exists(BASE_CSV_DIR):
    os.mkdir(BASE_CSV_DIR)
app.mount("/static", StaticFiles(directory=BASE_CSV_DIR), name="static")


class ResultBean(dict):

    def __init__(self, code=200, msg="success"):
        super().__init__(code=code, msg=msg)


@app.middleware("http")
async def http_filter(request: Request, call_next):
    try:
        response = await call_next(request)
    except BaseException as e:
        logger.error(str(e))
        traceback.print_exc()
        return JSONResponse(content=ResultBean(code=500, msg=str(e)))
    return response


@app.get("/")
def index():
    return RedirectResponse(url="/static/index.html")


@app.get("/system_info/")
async def system_info():
    return JSONResponse(content=ResultBean(msg=await sys_info()))


@app.get("/pid_img/")
async def pid_img(pid: int):
    img_bytes = await screenshot(pid, None)
    base64_encoded = base64.b64encode(img_bytes).decode('utf-8')
    return base64_encoded


@app.get("/get_pids/")
async def get_pids():
    return JSONResponse(content=ResultBean(msg=await pids()))


@app.get("/get_all_task/")
async def get_all_task():
    return JSONResponse(content=ResultBean(msg=await TaskCollection.get_all_task()))


@app.get("/run_task/")
async def run_task(request: Request, pid: int, pid_name: str, task_name: str):
    start_time = time.time()
    status = 0
    return_task_id, file_dir = await TaskCollection.create_task(pid, pid_name, BASE_CSV_DIR, task_name)

    task_process = TaskHandle(serialno=platform.node(), file_dir=file_dir,
                              task_id=return_task_id, platform=platform.system(), target_pid=pid)
    task_process.start()
    return JSONResponse(content=ResultBean())


@app.get("/stop_task/")
async def stop_task(request: Request, task_id: int):
    task = await TaskCollection.stop_task(task_id)
    try:
        TaskHandle.stop_handle(task.get("monitor_pid"))
    except BaseException as e:
        logger.error(e)
        traceback.print_exc()
    return JSONResponse(content=ResultBean())


@app.get("/result/")
async def task_result(request: Request, task_id: int):
    item_task = await TaskCollection.get_item_task(task_id)
    result = await DataCollect(item_task.get("file_dir")).get_all_data()
    return JSONResponse(content=ResultBean(msg=result))


@app.get("/task_status/")
async def task_task(request: Request, task_id: int):
    item_task = await TaskCollection.get_item_task(task_id)
    return JSONResponse(content=ResultBean(msg=item_task.status))  # 0未开始, 1 执行中 , 2 执行完成 3.暂停


@app.get("/delete_task/")
async def delete_task(request: Request, task_id: int):
    item = await TaskCollection.delete_task(task_id)
    if os.path.exists(item.get("file_dir")):
        try:
            shutil.rmtree(item.get("file_dir"))
        except:
            traceback.print_exc()
    return JSONResponse(content=ResultBean())


def open_url():
    time.sleep(2)
    webbrowser.open("http://127.0.0.1:20223")


def main():
    if __name__ == "__main__":
        import uvicorn
        multiprocessing.freeze_support()
        threading.Thread(target=open_url).start()
        logger.info("服务启动请访问: http://localhost:20223")
        uvicorn.run("pc_perf:app", host="0.0.0.0", port=20223, log_level="error", reload=False, workers=psutil.cpu_count())


if __name__ == "__main__":
    main()



# coding=utf-8
import asyncio
import os
from builtins import *
from multiprocessing.context import Process

import psutil

from dao import TaskCollection
from log import log as logger
from core.pc_tools import perf as pc_perf


class TaskHandle(Process):

    def __init__(self, serialno: str, target_pid: int, file_dir: str, task_id: int, platform: str):
        super(TaskHandle, self).__init__()
        self.serialno = serialno
        self.target_pid = target_pid
        self.file_dir = file_dir
        if not os.path.exists(self.file_dir):
            os.makedirs(self.file_dir)
        self.daemon = True
        self.task_id = task_id
        self.platform = platform  # platform.system()

    def start(self):
        logger.info("join task handle")
        super().start()

    def run(self):
        logger.info("join task handle run")
        asyncio.run(TaskCollection.set_task_running(self.task_id, self.pid))
        asyncio.run(pc_perf(self.target_pid, self.file_dir))

    @staticmethod
    def stop_handle(monitor_pid):
        logger.info("Stopping task handle and subprocesses...")
        # Terminate the pc_perf subprocess
        current_process = psutil.Process(monitor_pid)
        try:
            for child in current_process.children(recursive=True):
                child.terminate()
        except Exception as e:
            logger.error(e)
        finally:
            current_process.terminate()


if __name__ == '__main__':
    TaskHandle.stop_handle(15160)



from logging import getLogger, INFO, StreamHandler, Formatter
from concurrent_log_handler import ConcurrentRotatingFileHandler
import os

log = getLogger(__name__)
# Use an absolute path to prevent file rotation trouble.
logfile = os.path.abspath("log.log")
# Rotate log after reaching 512K, keep 5 old copies.
rotateHandler = ConcurrentRotatingFileHandler(logfile, "a", 512 * 1024 * 1024, 1)
log.addHandler(rotateHandler)
log.setLevel(INFO)
streamHandler = StreamHandler()
streamHandler.setLevel(INFO)
streamHandler.setFormatter(Formatter(
    '%(asctime)s - %(name)s - %(levelname)s - %(message)s'  # 日志格式
))
log.addHandler(streamHandler)


import asyncio
import datetime
import os
import platform
from sqlalchemy import Column, String, Integer, DateTime, select, or_
from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession
from sqlalchemy.orm import sessionmaker
from contextlib import asynccontextmanager
from sqlalchemy_serializer import SerializerMixin
from sqlalchemy.orm import declarative_base
from log import log as logger

logger.info("工作空间{0}".format(os.getcwd()))
db_path = os.path.join(os.getcwd(), "task.sqlite")
logger.info("db path {0}".format(db_path))
async_engine = create_async_engine('sqlite+aiosqlite:///{0}'.format(db_path), echo=False)
logger.info("current path {0}".format(os.getcwd()))
AsyncSessionLocal = sessionmaker(bind=async_engine, class_=AsyncSession, expire_on_commit=False)
Base = declarative_base()


@asynccontextmanager
async def async_connect():
    session = AsyncSessionLocal()
    try:
        logger.info("sql begin")
        yield session
        await session.commit()
        logger.info("sql success")
    except Exception as e:
        await session.rollback()
        raise e
    finally:
        logger.info("sql end")
        await session.close()


async def update_table_structure():
    async with async_engine.begin() as conn:
        # 反射现有的数据库结构
        await conn.run_sync(Base.metadata.create_all)


class Task(Base, SerializerMixin):
    __tablename__ = 'tasks'
    id = Column(Integer, primary_key=True, autoincrement=True)
    start_time = Column(DateTime, default=None)
    end_time = Column(DateTime, default=None)
    serialno = Column(String(255), default=None)
    status = Column(Integer)  # 0未开始, 1 执行中 , 2 执行完成 3.暂停
    file_dir = Column(String(255), default=None)  # 存储csv文件的路径
    target_pid = Column(Integer)  # 被测进程pid
    target_pid_name = Column(String(255), default=None)  # 被测进程pid 名称
    monitor_pid = Column(Integer, default=None)  # 当前任务运行的进程pid，任务执行的进程，里面有各个性能指标的线程
    platform = Column(String(50), default="win")  # win | mac | linux 任务
    name = Column(String(255), default=None)  # 任务名称


class TaskLabel(Base, SerializerMixin):
    __tablename__ = 'task_label'
    id = Column(Integer, primary_key=True, autoincrement=True)
    start_time = Column(DateTime, default=None)  # 标签开始
    end_time = Column(DateTime, default=None)  # 标签结束
    label_name = Column(String(255), default=None)  # 标签名称


class TaskCollection(object):

    @classmethod
    async def set_task_running(cls, task_id, monitor_pid):
        async with async_connect() as session:
            async with session.begin():
                result = await session.execute(select(Task).filter(Task.id == task_id))
                task = result.scalars().first()
                assert task, "NOT FIND TASK"
                assert task.status == 0, "TASK RUNNING FAIL, TASK STATUS IS {0}".format(task.status)
                task.status = 1
                task.monitor_pid = monitor_pid
                return task.to_dict()

    @classmethod
    async def get_all_task(cls):
        async with async_connect() as session:
            async with session.begin():
                result = await session.execute(select(Task))
                task = result.scalars().fetchall()
                result_list = [t.to_dict() for t in task]
                result_list.sort(key=lambda x: x.get("start_time"), reverse=True)
                return result_list

    @classmethod
    async def get_item_task(cls, task_id):
        async with async_connect() as session:
            async with session.begin():
                result = await session.execute(select(Task).filter(Task.id == task_id))
                task = result.scalars().first()
                assert task, "NOT FIND TASK"
                return task.to_dict()

    @classmethod
    async def create_task(cls, pid, pid_name, file_dir, name):
        async with async_connect() as session:
            async with session.begin():
                result = await session.execute(
                    select(Task).filter(Task.target_pid == pid).filter(or_(
                        Task.status == 0,
                        Task.status == 1,
                    )))
                task = result.scalars().first()
                assert not task, "MONITOR PID {0} TASK {1} IS RUN".format(pid, task.name)
                new_task = Task(start_time=datetime.datetime.now(), serialno=platform.node(), status=0,
                                target_pid=pid, platform=platform.system(), name=name, target_pid_name=pid_name)
                session.add(new_task)
                await session.flush()
                file_dir = os.path.join(file_dir, str(new_task.id))
                new_task.file_dir = file_dir
                await session.flush()
                return new_task.id, file_dir

    @classmethod
    async def stop_task(cls, task_id):
        async with async_connect() as session:
            async with session.begin():
                result = await session.execute(select(Task).filter(Task.id == task_id))
                task = result.scalars().first()
                assert task, "NOT FIND TASK"
                assert task.status != 0, "TASK NOT RUNNING, TASK STATUS IS {0}".format(task.status)
                task.status = 2
                task.end_time = datetime.datetime.now()
                return task.to_dict()

    @classmethod
    async def delete_task(cls, task_id):
        async with async_connect() as session:
            async with session.begin():
                result = await session.execute(select(Task).filter(Task.id == task_id))
                task = result.scalars().first()
                assert task, "NOT FIND TASK"
                assert task.status != 1, "TASK RUNNING NOT DELETE, TASK STATUS IS {0}".format(task.status)
                res = task.to_dict()
                await session.delete(task)
                return res

    @classmethod
    async def change_task_name(cls, task_id, new_name):
        async with async_connect() as session:
            async with session.begin():
                result = await session.execute(select(Task).filter(Task.id == task_id))
                task = result.scalars().first()
                assert task, "NOT FIND TASK"
                task.name = new_name
                return task.to_dict()


async def create_table():
    await update_table_structure()


asyncio.run(create_table())



import asyncio
import json
import platform
import subprocess
import threading
import time
import traceback
from io import BytesIO
import psutil
import pynvml
from pathlib import Path
from log import log
from core.monitor import Monitor

SUPPORT_GPU = True
try:
    pynvml.nvmlInit()
except:
    traceback.print_exc()
    log.info("本设备gpu获取不适配")
    SUPPORT_GPU = False
from PIL import ImageGrab


def print_json(msg):
    log.info(json.dumps(msg))


class WinFps(object):
    frame_que = list()
    single_instance = None
    fps_process = None

    def __init__(self, pid):
        self.pid = pid

    def __new__(cls, *args, **kwargs):
        if not cls.single_instance:
            cls.single_instance = super().__new__(cls)
        return cls.single_instance

    def fps(self):
        if not WinFps.fps_process:
            threading.Thread(target=self.start_fps_collect, args=(self.pid,)).start()
        if self.check_queue_head_frames_complete():
            return self.pop_complete_fps()

    @staticmethod
    def check_queue_head_frames_complete():
        if not WinFps.frame_que:
            return False
        head_time = int(WinFps.frame_que[0])
        end_time = int(WinFps.frame_que[-1])
        if head_time == end_time:
            return False
        return True

    @staticmethod
    def pop_complete_fps():
        head_time = int(WinFps.frame_que[0])
        complete_fps = []
        while int(WinFps.frame_que[0]) == head_time:
            complete_fps.append(WinFps.frame_que.pop(0))
        return complete_fps

    def start_fps_collect(self, pid):
        start_fps_collect_time = int(time.time())
        PresentMon = Path(__file__).parent.parent.joinpath("PresentMon.exe")
        res_terminate = subprocess.Popen(
            [PresentMon, "-process_id", str(pid), "-output_stdout", "-stop_existing_session"],
            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        WinFps.fps_process = res_terminate
        res_terminate.stdout.readline()
        while not res_terminate.poll():
            line = res_terminate.stdout.readline()
            if not line:
                try:
                    res_terminate.kill()
                except:
                    traceback.print_exc()
                break
            try:
                line = line.decode(encoding="utf-8")
                line_list = line.split(",")
                print("line ", line_list)
                WinFps.frame_que.append(start_fps_collect_time + round(float(line_list[7]), 7))
            except:
                time.sleep(1)
                traceback.print_exc()


async def sys_info():
    def real_func():
        current_platform = platform.system()
        computer_name = platform.node()
        res = {"platform": current_platform, "computer_name": computer_name, "time": time.time(),
               "cpu_cores": psutil.cpu_count(), "ram": "{0}G".format(int(psutil.virtual_memory().total / 1024 ** 3)),
               "rom": "{0}G".format(int(psutil.disk_usage('/').total / 1024 ** 3))}
        print_json(res)
        return res

    return await asyncio.wait_for(asyncio.to_thread(real_func), timeout=10)


async def pids():
    def real_func():
        process_list = []
        for proc in psutil.process_iter(attrs=['name', 'pid', 'cmdline', 'username']):
            try:
                if ("SYSTEM" not in str(proc.username())) and ("root" not in str(proc.username())):
                    process_list.append(
                        {"name": proc.info['name'], "pid": proc.info['pid'], "cmd": proc.info['cmdline'],
                         "username": proc.username()})
            except Exception as e:
                log.error(e)
        print_json(process_list)
        return process_list

    return await asyncio.wait_for(asyncio.to_thread(real_func), timeout=10)


async def screenshot(pid, save_dir):
    def real_func(pid, save_dir):
        start_time = int(time.time())
        if pid:
            window = None
            if platform.system() == "Windows":
                import ctypes
                import pygetwindow as gw
                def get_pid(hwnd):
                    pid = ctypes.wintypes.DWORD()
                    ctypes.windll.user32.GetWindowThreadProcessId(hwnd, ctypes.byref(pid))
                    return pid.value

                def get_window_by_pid(pid):
                    for window in gw.getAllWindows():
                        if get_pid(window._hWnd) == pid:
                            return window
                    return None

                window = get_window_by_pid(int(pid))
            if window:
                screenshot = ImageGrab.grab(
                    bbox=(window.left, window.top, window.left + window.width, window.top + window.height),
                    all_screens=True)
            else:
                screenshot = ImageGrab.grab(all_screens=True)
            if save_dir:
                dir_instance = Path(save_dir)
                screenshot_dir = dir_instance.joinpath("screenshot")
                screenshot_dir.mkdir(exist_ok=True)
                screenshot.save(screenshot_dir.joinpath(str(start_time) + ".png"), format="PNG")
            else:
                output_buffer = BytesIO()
                screenshot.save(output_buffer, format='PNG')
                output_buffer.seek(0)  # 重置缓冲区指针
                image_data = output_buffer.getvalue()
                return image_data

    return await asyncio.wait_for(asyncio.to_thread(real_func, pid, save_dir), timeout=10)


async def cpu(pid):
    def real_func(pid):
        start_time = int(time.time())
        proc = psutil.Process(pid=int(pid))
        cpu_usage = proc.cpu_percent(interval=1)
        cpu_count = psutil.cpu_count()
        res = {"cpu_usage": cpu_usage / cpu_count, "cpu_usage_all": cpu_usage,
               "cpu_core_num": cpu_count, "time": start_time}
        print_json(res)
        return res

    return await asyncio.wait_for(asyncio.to_thread(real_func, pid), timeout=10)


async def memory(pid):
    def real_func(pid):
        start_time = int(time.time())
        process = psutil.Process(int(pid))
        process_memory_info = process.memory_info()
        process_memory_usage = process_memory_info.rss / (1024 ** 2)  # In MB
        memory_info = {"process_memory_usage": process_memory_usage, "time": start_time}
        print_json(memory_info)
        return memory_info

    return await asyncio.wait_for(asyncio.to_thread(real_func, pid), timeout=10)


async def fps(pid):
    if platform.system() != "Windows":
        return {"type": "fps", "time": int(time.time())}
    frames = WinFps(pid).fps()
    if not frames:
        return frames
    res = {"type": "fps", "fps": len(frames), "frames": frames, "time": int(frames[0]) if frames else int(time.time())}
    print_json(res)
    return res


async def gpu(pid):
    def real_func(pid):
        pid = int(pid)
        start_time = int(time.time())
        if SUPPORT_GPU:
            device_count = pynvml.nvmlDeviceGetCount()
            res = None
            for i in range(device_count):
                handle = pynvml.nvmlDeviceGetHandleByIndex(i)
                processes = pynvml.nvmlDeviceGetComputeRunningProcesses(handle)
                for process in processes:
                    print(process)
                    if process.pid == pid:
                        gpu_Utilization = pynvml.nvmlDeviceGetUtilizationRates(handle)
                        gpu_utilization_percentage = gpu_Utilization.gpu  # GPU的计算使用率
                        res = {"gpu": gpu_utilization_percentage, "time": start_time}
                        print_json(res)
                        return res
            return res
        else:
            return {"time": start_time}

    return await asyncio.wait_for(asyncio.to_thread(real_func, pid), timeout=10)


async def process_info(pid):
    def real_func(pid):
        start_time = int(time.time())
        process = psutil.Process(int(pid))
        num_handles = None
        num_threads = None
        try:
            num_handles = process.num_handles()
        except:
            log.error(traceback.print_exc())
        try:
            num_threads = process.num_threads()
        except:
            log.error(traceback.print_exc())
        res = {"time": start_time}
        if num_handles: res["num_handles"] = num_handles
        if num_threads: res["num_threads"] = num_threads
        print_json(res)
        return res

    return await asyncio.wait_for(asyncio.to_thread(real_func, pid), timeout=10)


async def perf(pid, save_dir):
    monitors = {
        "cpu": Monitor(cpu,
                       pid=pid,
                       key_value=["time", "cpu_usage(%)", "cpu_usage_all(%)", "cpu_core_num(个)"],
                       save_dir=save_dir),
        "memory": Monitor(memory,
                          pid=pid,
                          key_value=["time", "process_memory_usage(M)"],
                          save_dir=save_dir),
        "process_info": Monitor(process_info,
                                pid=pid,
                                key_value=["time", "num_threads(个)", "num_handles(个)"],
                                save_dir=save_dir),
        "fps": Monitor(fps,
                       pid=pid,
                       key_value=["time", "fps(帧)", "frames"],
                       save_dir=save_dir),
        "gpu": Monitor(gpu,
                       pid=pid,
                       key_value=["time", "gpu(%)"],
                       save_dir=save_dir),
        "screenshot": Monitor(screenshot,
                              pid=pid,
                              save_dir=save_dir, is_out=False)
    }
    run_monitors = [monitor.run() for name, monitor in monitors.items()]
    await asyncio.gather(*run_monitors)



# coding=utf-8
from setuptools import setup, find_packages

setup(
    # 包的名称，通常与包的目录名称相同
    name='pc-perf',

    # 版本号，遵循语义化版本控制规则
    version='1.1.4',

    # 项目简短描述
    description='pc 进程性能测试平台，支持 windows / mac / linux 平台进程cpu、memory、fps（仅支持windows下OpenGL  DirectX 引擎应用）、gpu、thread_num、handle_num 等指标的实时监控和可视化展示',

    # 项目的URL，通常是项目主页或源代码仓库
    url='https://github.com/15525730080/pc_perf',

    # 作者
    author='范博洲',

    # 作者的电子邮件地址
    author_email='15525730080@163.com',

    # 包的许可证
    license='MIT',

    # 包的关键词
    keywords='pc fps cpu memory gpu monitor',

    # 定义项目所需的依赖
    install_requires=[
        "concurrent_log_handler==0.9.25",
        "fastapi==0.110.2",
        "numpy==1.23.5",
        "pandas==2.2.2",
        "Pillow==10.3.0",
        "psutil==5.9.8",
        "PyGetWindow==0.0.9",
        "pynvml==11.5.0",
        "SQLAlchemy==2.0.29",
        "SQLAlchemy_serializer==1.4.12",
        "starlette==0.37.2",
        "uvicorn==0.29.0",
        "aiosqlite==0.20.0"
    ],

    # 从包中自动寻找所有的子包和子模块
    py_modules=['dao', 'log', 'task_handle', 'util', 'pc_perf', 'core'],
    packages=['core'],
    
    # 包含数据文件，比如配置文件
    include_package_data=True,

    # 定义包中非.py文件的内容
    package_data={
        # 如果你的包中有数据文件，可以在这里指定
        '': ['../*.exe', '../test_result/*.html'],
    },

    # 指定Python版本要求
    python_requires='>=3.9',

    # # 指定包的入口点，用于命令行工具
    entry_points={
        'console_scripts': [
            'pc_perf=pc_perf:main',  # 这里的web-command是你为命令行工具定义的命令
        ],
    },

    # 指定分发文件的类别，例如："Programming Language :: Python :: 3"
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development :: Libraries',
    ],
)



import asyncio
import csv
import inspect
import json
import time
import traceback
from pathlib import Path

from log import log as logger


def print_json(data, *args, **kwargs):
    data_json = json.dumps(data)
    logger.info(data_json, *args, **kwargs)


class MonitorIter(object):

    def __init__(self, stop_event):
        self.stop_event = stop_event

    def __aiter__(self):
        return self

    def __anext__(self):
        if self.stop_event.is_set():
            future = asyncio.Future()
            future.set_result(None)
            return future
        elif not self.stop_event.is_set():
            raise StopAsyncIteration()


class Monitor(object):

    def __init__(self, func, **kwargs):
        super(Monitor, self).__init__()
        self.stop_event = asyncio.Event()
        self.func = func
        self.kwargs = kwargs
        self.stop_event.set()
        self.key_value = kwargs.get("key_value", [])
        self.name = self.func.__name__
        self.save_dir = kwargs.get("save_dir")
        self.is_out = self.kwargs.get("is_out", True)
        if self.is_out:
            dir_instance = Path(self.save_dir)
            if not dir_instance.exists():
                dir_instance.mkdir()
            csv_instance = dir_instance.joinpath(self.name + ".csv")
            self.csv_path = csv_instance.resolve()
            with open(self.csv_path, "w", encoding="utf-8", newline="") as f:
                csv_writer = csv.writer(f)
                csv_writer.writerow(self.key_value)
            self.key_value = [key.split("(")[0] for key in self.key_value]

    async def run(self):
        async for _ in MonitorIter(self.stop_event):
            before_func = time.time()
            param_names = inspect.signature(self.func).parameters.keys()
            params = {name: self.kwargs.get(name) for name in param_names}
            try:
                res = await self.func(**params)
                if self.is_out and res:
                    with open(self.csv_path, "a+", encoding="utf-8", newline="") as f:
                        csv_writer = csv.writer(f)
                        csv_writer.writerow([res.get(key, "") for key in self.key_value])
            except:
                logger.error(traceback.print_exc())
            finally:
                end_func = time.time()
                if interval_time := (int(end_func) - int(before_func)) <= 1:
                    await asyncio.sleep(interval_time)

    def stop(self):
        self.stop_event.clear()
